#!python
from flask import Flask, jsonify, abort, make_response, request

from sqlite3 import connect, Row

travaho = Flask(__name__)

@travaho.route('/')
def index():
    return make_response(jsonify("Workers of the world unite!"), 200)

@travaho.errorhandler(404)
def not_found(error):
    return make_response(jsonify({'error': 'Not found'}), 404)

@travaho.errorhandler(400)
def not_found(error):
    return make_response(jsonify({'error': 'Bad request'}), 400)

@travaho.route('/travaho/depts', methods=['GET'])
def get_depts():
    db = connect('travaho.db')
    db.row_factory = Row
    depts = db.execute("SELECT DISTINCT DEPT FROM EMPLOYEE") 
    result = {'depts' : [dept['DEPT'] for dept in depts]}
    return jsonify(result)

@travaho.route('/travaho/employees/<string:dept>', methods=['GET'])
def get_employees(dept):
    db = connect('travaho.db')
    db.row_factory = Row
    result = list()
    for employee in db.execute("SELECT ID, NAME, HIRED, SALARY, CITY FROM EMPLOYEE"):
        result.append({
            'id' : employee['ID'],
            'name' : employee['NAME'],
            'hired' : employee['HIRED'],
            'salary' : employee['SALARY'],
            'city' : employee['CITY']
        })

    db.close()

    return jsonify(result), 200

@travaho.route('/travaho/employee', methods=['POST'])
def create_employee():
    required_fields = set(['name','dept','salary','city'])
    optional_fields = set(['supervisor-id'])
    if (not request.json or not required_fields <= set(request.json.keys())): 
        abort(400) # bad request

    name = request.json['name']
    salary = request.json['salary']
    dept = request.json['dept']
    city = request.json['city']
    supervisor_id = request.json.get('supervisor-id',None)

    db = connect('travaho.db')
    db.row_factory = Row
    if (supervisor_id and 
        not (db.execute("SELECT ID FROM EMPLOYEE WHERE ID = ?", (str(supervisor_id),)).fetchone())):
        abort(400)
    

    c = db.execute("INSERT INTO EMPLOYEE(NAME, HIRED, DEPT, CITY, SALARY, ID_SUPERVISOR) " 
                   "VALUES (?,DATE(),?,?,?,?)", (name, dept, city, salary, supervisor_id))

    id = c.lastrowid
    db.commit()
    db.close()

    return jsonify({'employee-id':id}), 201

@travaho.route('/travaho/employee/<int:employee_id>', methods=['PUT'])
def update_employee(employee_id):
    if not request.json:
        abort(400) 


    db = connect('travaho.db')
    db.row_factory = Row
    employee = db.execute("SELECT * FROM EMPLOYEE WHERE ID = ?", (str(employee_id),)).fetchone()

    if not employee:
        abort(400)
        
    db.execute("UPDATE EMPLOYEE " +
               "SET (NAME,DEPT,CITY,SALARY,ID_SUPERVISOR) = (?,?,?,?,?)" +
               "WHERE ID = ?" , 
               (request.json.get('name',employee['NAME']),
                request.json.get('dept',employee['DEPT']),
                request.json.get('city',employee['CITY']),
                request.json.get('salary',employee['SALARY']),
                request.json.get('supervisor-id',employee['ID_SUPERVISOR']),
                str(employee_id)))
    db.commit()
    db.close()
    
    return jsonify({'employee-id':employee_id})


@travaho.route('/travaho/employee/<int:employee_id>', methods=['DELETE'])
def delete_employee(employee_id):
    db = connect('travaho.db')
    db.row_factory = Row
    employee = db.execute("SELECT * FROM EMPLOYEE WHERE ID = ?", (str(employee_id),)).fetchone()

    if not employee:
        return jsonify({'result' : 'OK'}), 304

    db.execute("DELETE FROM EMPLOYEE WHERE ID = ?", (str(employee_id),))
    db.commit()
    db.close()

    return jsonify({'result' : 'OK'})

if __name__ == '__main__':
    travaho.run(debug=True)

    
